import * as v from 'valibot';
import {
	Resource,
	Language,
	MediaType,
	Charset,
	ApplicationParams,
	ApplicationState,
	FALLBACK_LANGUAGE,
	FALLBACK_CHARSET, FALLBACK_MEDIA_TYPE,
} from './common';
import {BackendBuilder, createBackend, CreateBackendParams} from './backend';
import {ClientBuilder, createClient, CreateClientParams} from './client';

export interface ApplicationBuilder {
	mediaType(mediaType: MediaType): this;
	language(language: Language): this;
	charset(charset: Charset): this;
	resource<T extends v.BaseSchema>(resRaw: Resource<T>): this;
	createBackend(params: Omit<CreateBackendParams, 'app'>): BackendBuilder;
	createClient(params: Omit<CreateClientParams, 'app'>): ClientBuilder;
}

export const application = (appParams: ApplicationParams): ApplicationBuilder => {
	const appState: ApplicationState = {
		name: appParams.name,
		resources: new Set<Resource<any>>(),
		languages: new Set<Language>(),
		mediaTypes: new Set<MediaType>(),
		charsets: new Set<Charset>(),
	};

	appState.languages.add(FALLBACK_LANGUAGE);
	appState.charsets.add(FALLBACK_CHARSET);
	appState.mediaTypes.add(FALLBACK_MEDIA_TYPE);

	return {
		mediaType(serializerPair: MediaType) {
			appState.mediaTypes.add(serializerPair);
			return this;
		},
		charset(encodingPair: Charset) {
			appState.charsets.add(encodingPair);
			return this;
		},
		language(language: Language) {
			appState.languages.add(language);
			return this;
		},
		resource<T extends v.BaseSchema>(resRaw: Resource<T>) {
			appState.resources.add(resRaw);
			return this;
		},
		createBackend(params: Omit<CreateBackendParams, 'app'>) {
			return createBackend({
				...params,
				app: appState
			});
		},
		createClient(params: Omit<CreateClientParams, 'app'>) {
			return createClient({
				...params,
				app: appState
			});
		},
	};
};
