import {Language} from './language';
import {Charset} from './charset';
import {MediaType} from './media-type';

export * from './app';
export * from './charset';
export * from './media-type';
export * from './resource';
export * from './language';

export const FALLBACK_LANGUAGE = {
	name: 'en' as const,
	statusMessages: {
		unableToSerializeResponse: 'Unable To Serialize Response',
		unableToEncodeResponse: 'Unable To Encode Response',
		unableToInitializeResourceDataSource: 'Unable To Initialize $RESOURCE Data Source',
		unableToFetchResourceCollection: 'Unable To Fetch $RESOURCE Collection',
		unableToFetchResource: 'Unable To Fetch $RESOURCE',
		unableToDeleteResource: 'Unable To Delete $RESOURCE',
		languageNotAcceptable: 'Language Not Acceptable',
		encodingNotAcceptable: 'Encoding Not Acceptable',
		mediaTypeNotAcceptable: 'Media Type Not Acceptable',
		methodNotAllowed: 'Method Not Allowed',
		urlNotFound: 'URL Not Found',
		badRequest: 'Bad Request',
		ok: 'OK',
		resourceCollectionFetched: '$RESOURCE Collection Fetched',
		resourceFetched: '$RESOURCE Fetched',
		resourceNotFound: '$RESOURCE Not Found',
		deleteNonExistingResource: 'Delete Non-Existing $RESOURCE',
		resourceDeleted: '$RESOURCE Deleted',
		unableToDeserializeRequest: 'Unable To Deserialize Request',
		patchNonExistingResource: 'Patch Non-Existing $RESOURCE',
		unableToPatchResource: 'Unable To Patch $RESOURCE',
		invalidResourcePatch: 'Invalid $RESOURCE Patch',
		invalidResource: 'Invalid $RESOURCE',
		resourcePatched: '$RESOURCE Patched',
		resourceCreated: '$RESOURCE Created',
		resourceReplaced: '$RESOURCE Replaced',
		unableToGenerateIdFromResourceDataSource: 'Unable To Generate ID From $RESOURCE Data Source',
		unableToEmplaceResource: 'Unable To Emplace $RESOURCE',
		resourceIdNotGiven: '$RESOURCE ID Not Given',
		unableToCreateResource: 'Unable To Create $RESOURCE',
	},
	bodies: {
		languageNotAcceptable: [],
		encodingNotAcceptable: [],
		mediaTypeNotAcceptable: []
	},
} satisfies Language;

export const FALLBACK_CHARSET = {
	encode: (str: string) => Buffer.from(str, 'utf-8'),
	decode: (buf: Buffer) => buf.toString('utf-8'),
	name: 'utf-8' as const,
} satisfies Charset;

export const FALLBACK_MEDIA_TYPE = {
	serialize: (obj: unknown) => JSON.stringify(obj),
	deserialize: (str: string) => JSON.parse(str),
	name: 'application/json' as const,
} satisfies MediaType;
